import {TableProMaxProps, TableProMaxSlots} from "@/types/components/table";
import {
    EnterprisesUnitPagerQueryParams,
    EnterprisesUnitPagerVo, EnterprisesUnitSaveOrUpdateParams,
    PoliceUnitPagerVo
} from "@/types/views/unitManage/police/policeUnit.ts";
import {ref} from "vue";
import {FormExpose} from "ant-design-vue/es/form/Form";
import {ComponentExposed} from "vue-component-type-helpers";
import MapContainer from "@/components/aMap/MapContainer.vue";
import {FormProMaxItemOptions} from "@/types/components/form";
import {dictSelectNodes} from "@/config/dict.ts";
import {Button, message, Modal, Space} from "ant-design-vue";
import api from "@/axios";
import TableProMax from "@/components/table/TableProMax.vue";
import {deleteDataModal} from "@/components/tsx/ModalPro.tsx";
import {PageParams} from "@/types/hooks/useTableProMax.ts";
import FormProMax from "@/components/form/FormProMax.vue";

type _TableProps = TableProMaxProps<EnterprisesUnitPagerVo, EnterprisesUnitPagerQueryParams>;
type _FormType = EnterprisesUnitSaveOrUpdateParams & {
    contactPersonInfoName?: string;
    contactPersonInfoTelephone?: string
}
const saveOrUpdateEnterprisesUnit = (params: _FormType, callback: Function) => {
    const _formRef = ref<FormExpose>(null)
    const _mapRef = ref<ComponentExposed<typeof MapContainer>>(null)
    const _formParams = ref<_FormType>({...params})

    let city = '';

    const initMarker = (map: AMap.Map) => {
        //添加maker点 设置point
        const maker = new AMap.Marker({
            position: _formParams.value.point,
            draggable: true
        })
        maker.on("dragend", ({lnglat}) => {
            _formParams.value.point = lnglat
        })
        map.clearMap()
        map.add(maker)
        map.setFitView()
        console.log(123);
    }

    const _formOptions = ref<FormProMaxItemOptions<_FormType>>({
        name: {
            type: 'input',
            label: '单位名称',
            required: true,
        },
        type: {
            type: 'select',
            label: '单位类型',
            required: true,
            options: dictSelectNodes('EnterprisesUnitType')
        },
        administrativeDivisionCodes: {
            type: 'administrativeDivisionTree',
            label: '行政区划',
            required: true,
            componentsProps: {
                displayRender: ({labels}): string => {
                    city = labels[0]
                    return labels.join(' / ');
                }
            }
        },
        address: {
            type: 'inputTextArea',
            label: '详细地址',
        },
        map: {
            type: 'custom',
            label: '经纬度',
            customRender: () => <MapContainer
                ref={_mapRef}
                style={{width: '100%', height: '300px', position: 'relative'}}
                initCallback={(map) => {
                    AMap.plugin(['AMap.AutoComplete'], () => {
                        //@ts-ignore
                        const auto = new AMap.AutoComplete({
                            city,
                            input: "tipinput"
                        });
                        //注册监听，当选中某条记录时会触发
                        auto.on("select", (e) => {
                            //有些点位可能没有经纬度信息
                            if (!e.poi.location) {
                                message.error('所选点位没有经纬度信息 建议选则附近的手动移动!');
                                return
                            }
                            _formParams.value.point = e.poi.location
                            initMarker(map)
                        });
                    })
                    if (_formParams.value.point) {
                        initMarker(map)
                    }
                }}
            >
                <div style={{position: 'absolute', left: '10px', top: '10px', zIndex: 9999}}>
                    <input id={'tipinput'}
                           placeholder={'请输入详细地址'}
                           autocomplete="off"
                    />
                </div>
            </MapContainer>
        },
        contactPersonInfoName: {
            type: 'input',
            label: '联系人名称'
        },
        contactPersonInfoTelephone: {
            type: 'input',
            label: '联系人电话'
        },
        remark: {
            type: 'inputTextArea',
            label: '备注'
        }
    })
    Modal.confirm({
        title: params.snowFlakeId ? `【${params.name}】 信息编辑` : '新增企事业单位',
        width: 600,
        icon: ' ',
        centered: true,
        content: () => <FormProMax
            ref={_formRef}
            v-model:value={_formParams.value}
            formItemOptions={_formOptions.value}/>,
        onOk: async () => {
            await _formRef.value?.validate()
            const resp = await api.post('/enterprisesUnit/saveOrUpdate', {
                ..._formParams.value,
                contactPersonInfo: {
                    name: _formParams.value.contactPersonInfoName,
                    telephone: _formParams.value.contactPersonInfoTelephone
                }
            })
            message.success(resp.message)
            callback && callback()
        }
    })
}

export const showEnterprisesUnit = (policeUnitPagerVo: PoliceUnitPagerVo) => {
    const _tableRef = ref<ComponentExposed<typeof TableProMax>>(null)
    const _columns: _TableProps['columns'] = [
        {
            dataIndex: 'name',
            title: '名称'
        }, {
            dataIndex: 'type',
            title: '类型',
            customRender: ({text}) => text?.label
        }, {
            dataIndex: 'contactPersonInfo',
            title: '联系人',
            customRender: ({text}) => text?.name + "/" + text.telephone
        }, {
            dataIndex: 'province',
            title: '行政区划',
            customRender: ({record}) => [record.provinceName, record.cityName, record.districtsName, record.streetName].filter(Boolean).join("/")
        }, {
            dataIndex: 'address',
            title: '详细地址'
        }, {
            dataIndex: 'remark',
            title: '备注'
        }, {
            dataIndex: 'createTime',
            title: '创建时间'
        }, {
            dataIndex: 'opt',
            title: '操作',
            customRender: ({record}) => <Space>
                <Button
                    class="btn-warn"
                    onClick={() => saveOrUpdateEnterprisesUnit({
                        snowFlakeId: record.snowFlakeId,
                        policeUnitId: record.policeUnitId,
                        name: record.name,
                        type: record.type.value,
                        administrativeDivisionCodes: [record.province, record.city, record.districts, record.street].filter(Boolean),
                        address: record.address,
                        point: record.point,
                        contactPersonInfoName: record.contactPersonInfo?.name,
                        contactPersonInfoTelephone: record.contactPersonInfo?.telephone,
                        remark: record.remark
                    }, _tableRef.value?.requestGetTableData)}
                >编辑
                </Button>
                <Button class="btn-danger" onClick={() => deleteDataModal(record.name, async () => {
                    const resp = await api.delete('/enterprisesUnit/deleteById', {
                        enterprisesUnitId: record.snowFlakeId
                    })
                    message.success(resp.message)
                    await _tableRef.value?.requestGetTableData()
                })}>删除
                </Button>
            </Space>
        }
    ]
    const _reqApi: _TableProps["requestApi"] = (params) => {
        (params as PageParams<EnterprisesUnitPagerQueryParams>).params.policeUnitId = policeUnitPagerVo.snowFlakeId
        return api.post('/enterprisesUnit/pager', params)
    }
    Modal.info({
        title: `【${policeUnitPagerVo.name}】 管辖企事业单位`,
        width: '80%',
        content: () => <TableProMax
            ref={_tableRef}
            columns={_columns}
            requestApi={_reqApi}
            v-slots={{
                tableHeader: (_) => {
                    return <Space>
                        <Button
                            class="btn-success"
                            onClick={() => saveOrUpdateEnterprisesUnit({
                                name: undefined,
                                type: 'school',
                                policeUnitId: policeUnitPagerVo.snowFlakeId,
                                administrativeDivisionCodes: [policeUnitPagerVo.province, policeUnitPagerVo.city, policeUnitPagerVo.districts, policeUnitPagerVo.street].filter(Boolean)
                            }, _tableRef.value?.requestGetTableData)}
                        >新增
                        </Button>
                        <Button disabled>导入</Button>
                    </Space>
                }
            } as TableProMaxSlots<PoliceUnitPagerVo>}
        />
    })
}
